@@ -28,8 +28,8 @@
 
 GtkWidget             * link_properties      (GtkHTMLControlData *cd, gpointer *set_data);
 GtkWidget             * link_insert          (GtkHTMLControlData *cd, gpointer *set_data);
-void                    link_insert_cb       (GtkHTMLControlData *cd, gpointer  get_data);
-void                    link_apply_cb        (GtkHTMLControlData *cd, gpointer  get_data);
+gboolean                link_insert_cb       (GtkHTMLControlData *cd, gpointer  get_data);
+gboolean                link_apply_cb        (GtkHTMLControlData *cd, gpointer  get_data);
 void                    link_close_cb        (GtkHTMLControlData *cd, gpointer  get_data);
 
 #endif
