@@ -57,6 +57,7 @@ static void         spell_error_destroy     (SpellError *se);
 static void         move_spell_errors       (GList *spell_errors, guint offset, gint delta);
 static GList *      remove_spell_errors     (GList *spell_errors, guint offset, guint len);
 static void         remove_text_slaves      (HTMLObject *self);
+static void         html_text_change_attrs  (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index);
 
 /* void
 debug_spell_errors (GList *se)
@@ -148,8 +149,8 @@ copy (HTMLObject *s,
 	dest->font_style    = src->font_style;
 	dest->face          = g_strdup (src->face);
 	dest->color         = src->color;
-	dest->select_start  = src->select_start;
-	dest->select_length = src->select_length;
+	dest->select_start  = 0;
+	dest->select_length = 0;
 	dest->attr_list     = pango_attr_list_copy (src->attr_list);
 	dest->extra_attr_list = src->extra_attr_list ? pango_attr_list_copy (src->extra_attr_list) : NULL;
 
@@ -1122,7 +1123,11 @@ html_text_get_pango_info (HTMLText *text, HTMLPainter *painter)
 			attrs = pango_attr_list_copy (text->attr_list);
 		if (text->extra_attr_list)
 			pango_attr_list_splice (attrs, text->extra_attr_list, 0, text->text_len);
+		if (HTML_OBJECT (text)->parent && HTML_IS_CLUEFLOW (HTML_OBJECT (text)->parent) && painter->widget && GTK_IS_HTML (painter->widget)) {
+			GtkHTMLFontStyle parent_style = html_clueflow_get_default_font_style (HTML_CLUEFLOW (HTML_OBJECT (text)->parent));
 
+			html_text_change_attrs (attrs, parent_style, GTK_HTML (painter->widget)->engine, 0, text->text_bytes);
+		}
 		if (text->select_length) {
 			gchar *end;
 			gchar *start;
@@ -1358,7 +1363,7 @@ save_open_attrs (HTMLEngineSaveState *state, GSList *attrs)
 		case PANGO_ATTR_SIZE:
 			if (attr->klass == &html_pango_attr_font_size_klass) {
 				HTMLPangoAttrFontSize *size = (HTMLPangoAttrFontSize *) attr;
-				if ((size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3) {
+				if ((size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3 && (size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != 0) {
 					tag = g_strdup_printf ("<FONT SIZE=\"%d\">", size->style & GTK_HTML_FONT_STYLE_SIZE_MASK);
 					free_tag = TRUE;
 				}
@@ -1412,7 +1417,7 @@ save_close_attrs (HTMLEngineSaveState *state, GSList *attrs)
 		case PANGO_ATTR_SIZE:
 			if (attr->klass == &html_pango_attr_font_size_klass) {
 				HTMLPangoAttrFontSize *size = (HTMLPangoAttrFontSize *) attr;
-				if ((size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3)
+				if ((size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3 && (size->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != 0)
 					tag = "</FONT>";
 			}
 			break;
@@ -2915,7 +2920,10 @@ html_pango_attr_font_size_calc (HTMLPangoAttrFontSize *attr, HTMLEngine *e)
 	gint size, base_size, real_size;
 
 	base_size = (attr->style & GTK_HTML_FONT_STYLE_FIXED) ? e->painter->font_manager.fix_size : e->painter->font_manager.var_size;
-	size = (attr->style & GTK_HTML_FONT_STYLE_SIZE_MASK) - GTK_HTML_FONT_STYLE_SIZE_3;
+	if ((attr->style & GTK_HTML_FONT_STYLE_SIZE_MASK) != 0)
+		size = (attr->style & GTK_HTML_FONT_STYLE_SIZE_MASK) - GTK_HTML_FONT_STYLE_SIZE_3;
+	else
+		size = 0;
 	real_size = e->painter->font_manager.magnification * ((gdouble) base_size + (size > 0 ? (1 << size) : size) * base_size/8.0);
 
 	attr->attr_int.value = real_size;
@@ -3035,8 +3043,8 @@ html_text_get_style_conflicts (HTMLText *text, GtkHTMLFontStyle style, gint star
 	return conflicts;
 }
 
-void
-html_text_set_style_in_range (HTMLText *text, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index)
+static void
+html_text_change_attrs (PangoAttrList *attr_list, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index)
 {
 	PangoAttribute *attr;
 
@@ -3045,45 +3053,49 @@ html_text_set_style_in_range (HTMLText *text, GtkHTMLFontStyle style, HTMLEngine
 		attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
 		attr->start_index = start_index;
 		attr->end_index = end_index;
-		pango_attr_list_change (text->attr_list, attr);
+		pango_attr_list_change (attr_list, attr);
 	}
 
 	if (style & GTK_HTML_FONT_STYLE_ITALIC) {
 		attr = pango_attr_style_new (PANGO_STYLE_ITALIC);
 		attr->start_index = start_index;
 		attr->end_index = end_index;
-		pango_attr_list_change (text->attr_list, attr);
+		pango_attr_list_change (attr_list, attr);
 	}
 
 	if (style & GTK_HTML_FONT_STYLE_UNDERLINE) {
 		attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
 		attr->start_index = start_index;
 		attr->end_index = end_index;
-		pango_attr_list_change (text->attr_list, attr);
+		pango_attr_list_change (attr_list, attr);
 	}
 
 	if (style & GTK_HTML_FONT_STYLE_STRIKEOUT) {
 		attr = pango_attr_strikethrough_new (TRUE);
 		attr->start_index = start_index;
 		attr->end_index = end_index;
-		pango_attr_list_change (text->attr_list, attr);
+		pango_attr_list_change (attr_list, attr);
 	}
 
 	if (style & GTK_HTML_FONT_STYLE_FIXED) {
 		attr = pango_attr_family_new (e->painter->font_manager.fixed.face ? e->painter->font_manager.fixed.face : "Monospace");
 		attr->start_index = start_index;
 		attr->end_index = end_index;
-		pango_attr_list_change (text->attr_list, attr);
+		pango_attr_list_change (attr_list, attr);
 	}
 
-	/* size */
-	if ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) && ((style & GTK_HTML_FONT_STYLE_SIZE_MASK) != GTK_HTML_FONT_STYLE_SIZE_3)) {
-		attr = html_pango_attr_font_size_new (style);
-		html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
-		attr->start_index = start_index;
-		attr->end_index = end_index;
-		pango_attr_list_change (text->attr_list, attr);
-	}
+	/* always apply size so that we can zoom */
+	attr = html_pango_attr_font_size_new (style);
+	html_pango_attr_font_size_calc ((HTMLPangoAttrFontSize *) attr, e);
+	attr->start_index = start_index;
+	attr->end_index = end_index;
+	pango_attr_list_change (attr_list, attr);
+}
+
+void
+html_text_set_style_in_range (HTMLText *text, GtkHTMLFontStyle style, HTMLEngine *e, gint start_index, gint end_index)
+{
+	html_text_change_attrs (text->attr_list, style, e, start_index, end_index);
 }
 
 void
